package HTTP::Body::Builder;
use 5.008005;
use strict;
use warnings;

our $VERSION = "0.01";

1;
__END__

=encoding utf-8

=head1 NAME

HTTP::Body::Builder - HTTP::Body::Builder!

=head1 DESCRIPTION

HTTP::Body::Builder is a module set for building HTTP body.

See following modules.

=over 4

=item L<HTTP::Body::Builder::MultiPart>

=item L<HTTP::Body::Builder::UrlEncoded>

=back

=head1 LICENSE

Copyright (C) tokuhirom.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 AUTHOR

tokuhirom E<lt>tokuhirom@gmail.comE<gt>

=cut

